/*
 * jSite2 - Core.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.List;
import java.util.concurrent.Executor;


/**
 * Interface for the core.
 * 
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public interface Core {

	/**
	 * Adds the given listener to the list of registered listeners.
	 * 
	 * @param coreListener
	 *            The listener to add
	 */
	public void addCoreListener(CoreListener coreListener);

	/**
	 * Removes the given listener from the list of registered listeners.
	 * 
	 * @param coreListener
	 *            The listener to remove
	 */
	public void removeCoreListener(CoreListener coreListener);

	/**
	 * Returns a thread pool.
	 * 
	 * @return A thread pool
	 */
	public Executor getThreadPool();

	/**
	 * Adds the given node to the core.
	 * 
	 * @param node
	 *            The node to add
	 * @return <code>true</code> if the node was added, <code>false</code>
	 *         if it was not added because it was already known
	 * @throws UnknownHostException
	 *             if the hostname of the node can not be resolved
	 */
	public boolean addNode(Node node) throws UnknownHostException;

	/**
	 * Removes the given node from the core.
	 * 
	 * @param node
	 *            The node to remove
	 */
	public void removeNode(Node node);

	/**
	 * Returns the list of all configured nodes.
	 * 
	 * @return All configured nodes
	 */
	public List<Node> getNodes();

	/**
	 * Returns whether the core is currently connected to the given node.
	 * 
	 * @param node
	 *            The node to check
	 * @return <code>true</code> if the core is currently connected to the
	 *         node, <code>false</code> otherwise
	 */
	public boolean isNodeConnected(Node node);

	/**
	 * Starts the core.
	 */
	public void start();

	/**
	 * Stops the core.
	 */
	public void stop();

	/**
	 * Connects to the given node.
	 * 
	 * @param node
	 *            The node to connect to
	 */
	public void connectToNode(Node node);

	/**
	 * Disconnects from the given node.
	 * 
	 * @param node
	 *            The node to disconnect from
	 */
	public void disconnectFromNode(Node node);

	/**
	 * Creates a new project.
	 * 
	 * @throws IOException
	 *             if an I/O error occured communicating with the node
	 * @throws JSiteException
	 *             if there is a problem with the node
	 */
	public void createProject() throws IOException, JSiteException;

	/**
	 * Clones the given project. {@link CoreListener}s will be notified of the
	 * new clone via the {@link CoreListener#projectCloned(Project, Project)}
	 * event.
	 * 
	 * @param project
	 *            The project to clone
	 */
	public void cloneProject(Project project);

	/**
	 * Removes the given project.
	 * 
	 * @param project
	 *            The project to remove
	 */
	public void removeProject(Project project);

	/**
	 * Returns a list of all projects.
	 * 
	 * @return A list of all projects
	 */
	public List<Project> getProjects();

}
