/*
 * jSite2 - Insert.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

/**
 * Represents a currently running or past insert.
 * 
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public class Insert {

	/** The project that was inserted. */
	private final Project project;

	/** The node the project was inserted to. */
	private final Node node;

	/** The start time of the insert. */
	private final long startTime;

	/** The end time of the insert. */
	private long endTime;

	/** Whether the insert was successful. */
	private boolean success;

	/**
	 * Creates a new insert.
	 * 
	 * @param project
	 *            The project that is inserted
	 * @param node
	 *            The node the project is inserted to
	 * @param startTime
	 *            The time the insert was started
	 */
	public Insert(Project project, Node node, long startTime) {
		this.project = project;
		this.node = node;
		this.startTime = startTime;
	}

	/**
	 * Returns the project that is inserted.
	 * 
	 * @return The inserted project
	 */
	public Project getProject() {
		return project;
	}

	/**
	 * Returns the node the project is inserted to.
	 * 
	 * @return The node the project is inserted to
	 */
	public Node getNode() {
		return node;
	}

	/**
	 * Returns the start time of the insert.
	 * 
	 * @return The start time of the insert
	 */
	public long getStartTime() {
		return startTime;
	}

	/**
	 * Returns the end time of the insert. If the insert has not yet finished,
	 * <code>-1</code> is returned.
	 * 
	 * @return The end time of the insert, or <code>-1</code> if the insert is
	 *         still running
	 */
	public long getEndTime() {
		return endTime;
	}

	/**
	 * Sets the end time of the insert.
	 * 
	 * @param endTime
	 *            The end time of the insert
	 */
	public void setEndTime(long endTime) {
		this.endTime = endTime;
	}

	/**
	 * Returns whether the insert was successful. When the project has not yet
	 * finished, i.e. {@link #getEndTime()} returns <code>-1</code>, the
	 * return value of this method is undefined.
	 * 
	 * @return <code>true</code> if the insert finished successfully,
	 *         <code>false</code> otherwise
	 */
	public boolean isSuccess() {
		return success;
	}

	/**
	 * Sets whether the insert finished successfully.
	 * 
	 * @param success
	 *            <code>true</code> if the insert finished successfully,
	 *            <code>false</code> otherwise
	 */
	public void setSuccess(boolean success) {
		this.success = success;
	}

}
