/*
 * jSite2 - MainWindow.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import net.pterodactylus.jsite.core.Node;
import net.pterodactylus.jsite.core.Project;
import net.pterodactylus.jsite.i18n.I18n;
import net.pterodactylus.jsite.i18n.I18nable;
import net.pterodactylus.jsite.i18n.gui.I18nAction;
import net.pterodactylus.jsite.i18n.gui.I18nMenu;
import net.pterodactylus.jsite.main.Version;
import net.pterodactylus.util.image.IconLoader;
import net.pterodactylus.util.logging.Logging;
import net.pterodactylus.util.swing.StatusBar;
import net.pterodactylus.util.swing.SwingUtils;

/**
 * Defines the main window of the application.
 * 
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public class MainWindow extends JFrame implements WindowListener, I18nable, PropertyChangeListener {

	/** Logger. */
	@SuppressWarnings("unused")
	private static final Logger logger = Logging.getLogger(MainWindow.class.getName());

	/** The swing interface that receives all actions. */
	private final SwingInterface swingInterface;

	/** The status bar. */
	private StatusBar statusBar = new StatusBar();

	/** Timer for clearing the status bar. */
	private Timer statusBarClearTimer = new Timer("StatusBar Cleaner", true);

	/** Object for status bar clearing ticker event. */
	private TimerTask statusBarClearTimerTask;

	/** Delay (in seconds) after which to clear status bar. */
	private int statusBarClearDelay = 5000;

	/** The icon for offline nodes. */
	private Icon offlineIcon;

	/** The icon for online nodes. */
	private Icon onlineIcon;

	/** The icon for error nodes. */
	private Icon errorIcon;

	/** The content pane. */
	private JPanel contentPane = new JPanel(new BorderLayout(12, 12));

	/** The jSite menu. */
	private I18nMenu jSiteMenu;

	/** The node menu. */
	private I18nMenu nodeMenu;

	/** The language menu. */
	private I18nMenu languageMenu;

	/** The about menu. */
	private I18nMenu helpMenu;

	/** The tabbed project pane. */
	private JTabbedPane projectPane;

	/** The project overview panel. */
	private JPanel projectOverviewPanel;

	/** Maps from node to menus. */
	private final Map<Node, JMenu> nodeMenus = new HashMap<Node, JMenu>();

	/** Maps from nodes to node panels. */
	private final Map<Node, NodeLabel> nodeLabels = new HashMap<Node, NodeLabel>();

	/**
	 * Creates a new main window that redirects all actions to the given swing
	 * interface.
	 * 
	 * @param swingInterface
	 *            The swing interface to receive all actions
	 */
	public MainWindow(SwingInterface swingInterface) {
		super("jSite " + Version.getVersion());
		this.swingInterface = swingInterface;
		initWindow();
		setPreferredSize(new Dimension(480, 280));
		pack();
		SwingUtils.center(this);
		I18n.registerI18nable(this);
		addWindowListener(this);
		setIconImage(IconLoader.loadImage("/jSite-frame-icon.png"));
	}

	//
	// ACCESSORS
	//

	/**
	 * Sets the text of the status bar.
	 * 
	 * @param text
	 *            The text of the status bar
	 */
	public void setStatusBarText(String text) {
		statusBar.setText(text);
		synchronized (statusBar) {
			if (statusBarClearTimerTask != null) {
				statusBarClearTimerTask.cancel();
			}
			statusBarClearTimerTask = new TimerTask() {

				@SuppressWarnings("synthetic-access")
				@Override
				public void run() {
					statusBar.setText("\u00a0");
				}

			};
			statusBarClearTimer.schedule(statusBarClearTimerTask, statusBarClearDelay);
		}
	}

	/**
	 * Returns the status bar clear delay (in milliseconds).
	 * 
	 * @return The status bar clear delay
	 */
	public int getStatusBarClearDelay() {
		return statusBarClearDelay;
	}

	/**
	 * Sets the status bar clear delay (in milliseconds).
	 * 
	 * @param statusBarClearDelay
	 *            The status bar clear delay
	 */
	public void setStatusBarClearDelay(int statusBarClearDelay) {
		this.statusBarClearDelay = statusBarClearDelay;
	}

	/**
	 * Sets whether the advanced mode is activated.
	 * 
	 * @param advancedMode
	 *            <code>true</code> if the advanced mode is activated,
	 *            <code>false</code> if the simple mode is activated
	 */
	public void setAdvancedMode(boolean advancedMode) {
		/* doesn’t do anything right now. */
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Container getContentPane() {
		return contentPane;
	}

	/**
	 * Returns the currently selected project.
	 * 
	 * @return The currently selected project
	 */
	public Project getSelectedProject() {
		return null;
	}

	/**
	 * Sets the given node to the “online” state.
	 * 
	 * @param node
	 *            The node to set online
	 */
	public void setOnline(Node node) {
		nodeLabels.get(node).setOnline();
	}

	/**
	 * Sets the given node to the “offline” state in the status bar.
	 * 
	 * @param node
	 *            The node to set offline
	 */
	public void setOffline(Node node) {
		nodeLabels.get(node).setOffline();
	}

	/**
	 * Sets the given node to the “error” state in the status bar.
	 * 
	 * @param node
	 *            The node to set the error state for
	 */
	public void setError(Node node) {
		nodeLabels.get(node).setError();
	}

	//
	// ACTIONS
	//

	/**
	 * Adds a node to the menu.
	 * 
	 * @param node
	 *            The node to add
	 */
	void addNode(Node node) {
		JMenu newNodeMenu = new JMenu(node.getName());
		nodeMenus.put(node, newNodeMenu);
		newNodeMenu.add(swingInterface.getNodeConnectAction(node));
		newNodeMenu.add(swingInterface.getNodeDisconnectAction(node));
		newNodeMenu.addSeparator();
		newNodeMenu.add(swingInterface.getNodeEditAction(node));
		newNodeMenu.add(swingInterface.getNodeDeleteAction(node));
		nodeMenu.add(newNodeMenu);
		NodeLabel nodeLabel = new NodeLabel(swingInterface, node, onlineIcon, offlineIcon, errorIcon);
		nodeLabels.put(node, nodeLabel);
		statusBar.addSideComponent(nodeLabel);
		for (int tabIndex = 0, tabCount = projectPane.getTabCount(); tabIndex < tabCount; tabIndex++) {
			Component tabComponent = projectPane.getComponentAt(tabIndex);
			if (!(tabComponent instanceof ProjectPanel)) {
				continue;
			}
			((ProjectPanel) tabComponent).addNode(node);
		}
		node.addPropertyChangeListener(this);
	}

	/**
	 * Removes a node from the menu.
	 * 
	 * @param node
	 *            The node to remove
	 */
	void removeNode(Node node) {
		nodeMenu.remove(nodeMenus.remove(node));
		statusBar.removeSideComponent(nodeLabels.remove(node));
		for (int tabIndex = 0, tabCount = projectPane.getTabCount(); tabIndex < tabCount; tabIndex++) {
			Component tabComponent = projectPane.getComponentAt(tabIndex);
			if (!(tabComponent instanceof ProjectPanel)) {
				continue;
			}
			((ProjectPanel) tabComponent).addNode(node);
		}
		node.removePropertyChangeListener(this);
	}

	/**
	 * Adds a project to the project pane.
	 * 
	 * @param project
	 *            The project to add
	 * @param switchToProject
	 *            <code>true</code> to switch to the new panel,
	 *            <code>false</code> to not change the current panel
	 */
	void addProject(Project project, boolean switchToProject) {
		ProjectPanel projectPanel = new ProjectPanel(swingInterface, project);
		int newTabIndex = projectPane.getTabCount();
		projectPane.add(project.getName(), projectPanel);
		projectPane.setToolTipTextAt(newTabIndex, project.getDescription());
		project.addPropertyChangeListener(this);
		if (switchToProject) {
			projectPane.setSelectedIndex(newTabIndex);
			while (project.getBasePath().length() == 0) {
				JOptionPane.showMessageDialog(this, I18n.get("mainWindow.information.changeProjectBasePath.message"), I18n.get("mainWindow.information.changeProjectBasePath.title"), JOptionPane.INFORMATION_MESSAGE);
				projectPanel.changeBasePath();
			}
		}
	}

	/**
	 * Removes the pane containing the given project.
	 * 
	 * @param project
	 *            The project whose pane to remove
	 */
	void removeProject(Project project) {
		int projectIndex = getProjectIndex(project);
		projectPane.remove(projectIndex);
	}

	/**
	 * @param project
	 */
	void projectInsertStarted(Project project) {
		int projectIndex = getProjectIndex(project);
		if (projectIndex == -1) {
			return;
		}
		projectPane.setTitleAt(projectIndex, I18n.get("projectPanel.title.starting", project.getName()));
	}

	/**
	 * @param project
	 * @param totalBlocks
	 * @param requiredBlocks
	 * @param successfulBlocks
	 * @param failedBlocks
	 * @param fatallyFailedBlocks
	 * @param finalizedTotal
	 */
	void projectInsertProgressed(Project project, int totalBlocks, int requiredBlocks, int successfulBlocks, int failedBlocks, int fatallyFailedBlocks, boolean finalizedTotal) {
		int projectIndex = getProjectIndex(project);
		if (projectIndex == -1) {
			return;
		}
		projectPane.setTitleAt(projectIndex, I18n.get("projectPanel.title.progress", project.getName(), requiredBlocks / (double) successfulBlocks));
	}

	/**
	 * @param project
	 */
	void projectInsertGeneratedURI(Project project) {
		/* TODO - update panel. */
	}

	/**
	 * @param project
	 * @param success
	 */
	void projectInsertFinished(Project project, boolean success) {
		int projectIndex = getProjectIndex(project);
		if (projectIndex == -1) {
			return;
		}
		projectPane.setTitleAt(projectIndex, project.getName());
	}

	//
	// PRIVATE METHODS
	//

	/**
	 * Returns the index of the project panel that contains the given project.
	 * 
	 * @param project
	 *            The wanted project
	 * @return The index of {@link #projectPane}’s tab that contains the given
	 *         project, or <code>-1</code> if the project can not be found
	 */
	private int getProjectIndex(Project project) {
		int tabCount = projectPane.getTabCount();
		for (int tabIndex = 1; tabIndex < tabCount; tabIndex++) {
			Component tabComponent = projectPane.getComponentAt(tabIndex);
			if (tabComponent instanceof ProjectPanel) {
				if (((ProjectPanel) tabComponent).getProject() == project) {
					return tabIndex;
				}
			}
		}
		return -1;
	}

	/**
	 * Initializes the window by creating all its components.
	 */
	private void initWindow() {
		onlineIcon = IconLoader.loadIcon("/node-online.png");
		offlineIcon = IconLoader.loadIcon("/node-offline.png");
		errorIcon = IconLoader.loadIcon("/node-error.png");

		JMenuBar menuBar = new JMenuBar();

		jSiteMenu = new I18nMenu("mainWindow.menu.jSite");
		menuBar.add(jSiteMenu);

		jSiteMenu.add(new FixedJMenuItem(swingInterface.getConfigureAction()));
		jSiteMenu.addSeparator();
		jSiteMenu.add(new FixedJMenuItem(swingInterface.getImportConfigAction()));
		jSiteMenu.addSeparator();
		jSiteMenu.add(new FixedJMenuItem(swingInterface.getQuitAction()));

		nodeMenu = new I18nMenu("mainWindow.menu.node");
		menuBar.add(nodeMenu);

		nodeMenu.add(new FixedJMenuItem(swingInterface.getAddNodeAction()));
		nodeMenu.addSeparator();

		languageMenu = new I18nMenu("mainWindow.menu.language");
		menuBar.add(languageMenu);

		for (I18nAction languageAction : swingInterface.getLanguageActions()) {
			languageMenu.add(new FixedJMenuItem(languageAction));
		}

		menuBar.add(Box.createHorizontalGlue());

		helpMenu = new I18nMenu("mainWindow.menu.help");
		menuBar.add(helpMenu);

		helpMenu.add(new FixedJMenuItem(swingInterface.getHelpAboutAction()));

		setJMenuBar(menuBar);

		JToolBar toolBar = new JToolBar(I18n.get("mainWindow.toolbar.name"));
		toolBar.add(swingInterface.getConfigureAction());
		toolBar.add(swingInterface.getQuitAction());
		toolBar.addSeparator();
		toolBar.add(swingInterface.getAddNodeAction());
		super.getContentPane().add(toolBar, BorderLayout.PAGE_START);

		super.getContentPane().add(contentPane, BorderLayout.CENTER);

		addWindowListener(new WindowAdapter() {

			/**
			 * {@inheritDoc}
			 */
			@SuppressWarnings("synthetic-access")
			@Override
			public void windowClosing(WindowEvent windowEvent) {
				swingInterface.getQuitAction().actionPerformed(null);
			}
		});

		initComponents();
	}

	/**
	 * Initializes all components of this window.
	 */
	private void initComponents() {
		super.getContentPane().add(statusBar, BorderLayout.PAGE_END);
		contentPane.setBorder(new EmptyBorder(12, 12, 12, 12));

		projectPane = new JTabbedPane(SwingConstants.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);
		contentPane.add(projectPane, BorderLayout.CENTER);

		projectOverviewPanel = new JPanel(new GridBagLayout());
		projectOverviewPanel.setName(I18n.get("mainWindow.pane.overview.title"));
		projectPane.add(projectOverviewPanel);
		projectOverviewPanel.setBorder(new EmptyBorder(12, 12, 12, 12));
		JButton addProjectButton = new JButton(swingInterface.getAddProjectAction());
		projectOverviewPanel.add(addProjectButton, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
	}

	//
	// INTERFACE I18nable
	//

	/**
	 * {@inheritDoc}
	 */
	public void updateI18n() {
		swingInterface.getConfigureAction().updateI18n();
		swingInterface.getImportConfigAction().updateI18n();
		swingInterface.getQuitAction().updateI18n();
		swingInterface.getAddNodeAction().updateI18n();
		swingInterface.getAddProjectAction().updateI18n();
		swingInterface.getHelpAboutAction().updateI18n();
		jSiteMenu.updateI18n();
		nodeMenu.updateI18n();
		languageMenu.updateI18n();
		for (Node node : swingInterface.getNodes()) {
			swingInterface.getNodeConnectAction(node).updateI18n();
			swingInterface.getNodeDisconnectAction(node).updateI18n();
			swingInterface.getNodeEditAction(node).updateI18n();
			swingInterface.getNodeDeleteAction(node).updateI18n();
		}
		for (Project project : swingInterface.getProjects()) {
			swingInterface.getCloneProjectAction(project).updateI18n();
			swingInterface.getDeleteProjectAction(project).updateI18n();
		}
		for (I18nAction languageAction : swingInterface.getLanguageActions()) {
			languageAction.updateI18n();
		}
		helpMenu.updateI18n();
		getJMenuBar().revalidate();
		projectPane.setTitleAt(0, I18n.get("mainWindow.pane.overview.title"));
		for (int componentIndex = 0; componentIndex < projectPane.getTabCount(); componentIndex++) {
			Component tabComponent = projectPane.getComponentAt(componentIndex);
			if (tabComponent instanceof ProjectPanel) {
				((ProjectPanel) tabComponent).updateI18n();
			}
		}
	}

	//
	// INTERFACE WindowListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void windowActivated(WindowEvent e) {
		/* do nothing. */
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowClosed(WindowEvent e) {
		/* do nothing. */
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowClosing(WindowEvent e) {
		swingInterface.getQuitAction().actionPerformed(null);
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowDeactivated(WindowEvent e) {
		/* do nothing. */
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowDeiconified(WindowEvent e) {
		/* do nothing. */
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowIconified(WindowEvent e) {
		/* do nothing. */
	}

	/**
	 * {@inheritDoc}
	 */
	public void windowOpened(WindowEvent e) {
		/* do nothing. */
	}

	//
	// INTERFACE PropertyChangeListener
	//

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
		Object eventSource = propertyChangeEvent.getSource();
		String propertyName = propertyChangeEvent.getPropertyName();
		if (eventSource instanceof Project) {
			/* if a project was changed, update the tab title and tooltip. */
			if (Project.PROPERTY_NAME.equals(propertyName) || Project.PROPERTY_DESCRIPTION.equals(propertyName)) {
				Project project = (Project) eventSource;
				int projectIndex = getProjectIndex(project);
				if (projectIndex != -1) {
					projectPane.setTitleAt(projectIndex, project.getName());
					projectPane.setToolTipTextAt(projectIndex, project.getDescription());
					projectPane.repaint();
				}
			}
		} else if (eventSource instanceof Node) {
			if (propertyName.equals(Node.PROPERTY_NAME)) {
				Node changedNode = (Node) eventSource;
				nodeMenus.get(changedNode).setText(changedNode.getName());
			}
		}
	}

}
